<?php
/**
 * wpJacker
 *
 * @package   wpJackerPro
 * @author    Chad Wyatt <chad@chadwyatt.com>
 * @license   commercial
 * @link      http://chadwyatt.com
 * @copyright 2014 Chad Wyatt
 */
//error_reporting(E_ALL);
class WP_JackerPro_Admin {

	protected static $instance = null;
	protected $plugin_screen_hook_suffix = null;


	private function __construct() {

		$plugin = WP_JackerPro::get_instance();
		$this->plugin_slug = $plugin->get_plugin_slug();

		
		// Load admin style sheet and JavaScript.
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		//Add ajax calls
		add_action('wp_ajax_wpjacker_jackit_form', array( $this, 'wpjacker_jackit_form' ) );
		add_action('wp_ajax_wpjacker_jackit_now', array( $this, 'wpjacker_jackit_now' ) );
		
		add_action('wp_ajax_wpjregister_parent_js', array( $this, 'wpjacker_parent_js' ) );
		add_action('wp_ajax_nopriv_wpjregister_parent_js', array( $this, 'wpjacker_parent_js' ) );
		
		add_action('wp_ajax_wpjregister_parent_html', array( $this, 'wpjacker_parent_html' ) );
		add_action('wp_ajax_nopriv_wpjregister_parent_html', array( $this, 'wpjacker_parent_html' ) );
		
		add_action('wp_ajax_wpjregister_record_optin', array( $this, 'record_optin' ) );
		add_action('wp_ajax_nopriv_wpjregister_record_optin', array( $this, 'record_optin' ) );
		
		add_action('wp_ajax_wpjregister_record_like', array( $this, 'record_like' ) );
		add_action('wp_ajax_nopriv_wpjregister_record_like', array( $this, 'record_like' ) );
		
		add_action('wp_ajax_wpjregister_record_oto_click', array( $this, 'record_oto_click' ) );
		add_action('wp_ajax_nopriv_wpjregister_record_oto_click', array( $this, 'record_oto_click' ) );
		
		add_action('wp_ajax_wpjregister_record_activation', array( $this, 'record_activation' ) );
		add_action('wp_ajax_nopriv_wpjregister_record_activation', array( $this, 'record_activation' ) );
		
		
		add_action('wp_ajax_wpjacker_delete_profile', array( $this, 'delete_profile' ) );
		add_action('wp_ajax_wpjacker_edit_profile', array( $this, 'edit_profile' ) );
		add_action('wp_ajax_wpjacker_save_profile', array( $this, 'save_profile' ) );
		add_action('wp_ajax_wpjacker_list_profiles', array( $this, 'list_profiles' ) );
		add_action('wp_ajax_wpjacker_list_plugins', array( $this, 'list_plugins' ) );
		add_action('wp_ajax_wpjacker_edit_plugin_modal', array( $this, 'edit_plugin_modal' ) );
		add_action('wp_ajax_wpjacker_save_plugin', array( $this, 'save_plugin' ) );
		add_action('wp_ajax_wpjacker_delete_plugin', array( $this, 'delete_plugin' ) );
		add_action('wp_ajax_wpjacker_toggle_plugin_active', array( $this, 'toggle_plugin_active' ) ) ;
		
		
		add_action('wp_ajax_wpjregister_admin_notice', array( $this, 'wpjregister_admin_notice' ) );
		add_action('wp_ajax_nopriv_wpjregister_admin_notice', array( $this, 'wpjregister_admin_notice' ) );
		
		
		//add link to listed plugins
		//add_filter( 'plugin_action_links', array( $this, 'add_action_links' ) );
		
		//register post types
		add_action( 'init', array($this, 'register_post_types' ) );
		
		//add menu item
		add_action( 'admin_menu', array( $this, 'add_plugin_admin_menu' ) );
		
	}

	public static function get_instance() {

		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

	public function enqueue_admin_scripts() {
		wp_enqueue_script( $this->plugin_slug . '-admin-script', plugins_url( 'assets/js/admin.js', __FILE__ ), array( 'jquery' ) );
	}


	/**
	 * Register the administration menu for this plugin into the WordPress Dashboard menu.
	 *
	 * @since    1.0.0
	 */
	public function add_plugin_admin_menu() {

		/*
		 * Add a settings page for this plugin to the Settings menu.
		 *
		 * NOTE:  Alternative menu locations are available via WordPress administration menu functions.
		 *
		 *        Administration Menus: http://codex.wordpress.org/Administration_Menus
		 *
		 * @TODO:
		 *
		 * - Change 'Page Title' to the title of your plugin admin page
		 * - Change 'Menu Text' to the text for menu item for the plugin settings page
		 * - Change 'manage_options' to the capability you see fit
		 *   For reference: http://codex.wordpress.org/Roles_and_Capabilities
		 */
		/*$this->plugin_screen_hook_suffix = add_options_page(
			__( 'Video SMS Marketing Campaigns', $this->plugin_slug ),
			__( 'Menu Text', $this->plugin_slug ),
			'manage_options',
			$this->plugin_slug,
			array( $this, 'display_plugin_admin_page' )
		);*/
		
		//add menu item
		$page_title = "wpJacker Pro";
		$menu_title = "wpJacker Pro";
		$capability = "manage_options";
		$menu_slug = $this->plugin_slug;
		$function = array( $this, 'display_plugin_admin_page' );
		//$icon_url = plugins_url( 'wpjacker/admin/assets/Ninja-Icon-20a.png' );
		$icon_url = plugins_url( 'assets/Ninja-Icon-20a.png', __FILE__ );
		//$position = "";
		add_menu_page( $page_title, $menu_title, $capability, $menu_slug, $function, $icon_url, $position );
		

	}
	/**
	 * Render the settings page for this plugin.
	 *
	 * @since    1.0.0
	 */
	public function display_plugin_admin_page() {
		include_once( 'views/admin.php' );
	}


	public function register_post_types(){
		//$this->register_twilio_post_type();
		
		//wpjacker_profile
		$args = array( 
			'public' => false,
			'label' => 'wpJacker Profile'
		); 
		register_post_type( 'wpjacker_profile' , $args );
		
		//wpjacker_plugin
		$args = array( 
			'public' => false,
			'label' => 'wpJacker Plugin'
		); 
		register_post_type( 'wpjacker_plugin' , $args );
		
	}
	
	public function list_profiles(){
		include_once('views/admin-list-campaigns.php');
		die();
	}
	
	public function list_plugins(){
		include_once('views/admin-list-plugins.php');
		die();	
	}
	
	public function edit_plugin_modal(){
		include_once('views/edit-plugin-modal-content.php');
		die();	
	}
	
	public function save_plugin(){
		print_r($_REQUEST);
		
		if($_REQUEST['wpjacker_plugin_id'] > 0){
			//wp_update_post( 
			$ID = $_REQUEST['wpjacker_plugin_id'];
			
		} else {
			$args = array(
				'post_title' => $_REQUEST['wpjacker_plugin_name'],
				'post_type' => 'wpjacker_plugin',
				'post_status' => 'publish'
			);
			$ID = wp_insert_post( $args );
			update_post_meta($ID, 'plugin_file', $_REQUEST['wpjacker_plugin_file']);
		
		}
		
		update_post_meta($ID, 'profile_id', $_REQUEST['wpjacker_profile_id']);
		update_post_meta($ID, 'active', $_REQUEST['wpjacker_plugin_active']);
		
		die();
	}
	
	public function wpjacker_jackit_form() {
		//error_reporting(E_ALL);
		include_once('views/admin-jackit-form.php');
		die();
	}
	
	public function wpjacker_jackit_now(){
		//print_r($_REQUEST);
		//die();
		
		//save all the settings to the db for this profile
		//$ID = $this->save_profile();
		//echo $ID;
		//die();
		
		//plugin ID
		$ID = $_REQUEST['ID'];
		$file = $_REQUEST['plugin_file'];
		$parts = pathinfo($file);
		
		$plugin_path =  plugin_dir_path( __FILE__ );
		
		$this_parts = explode("/", $plugin_path);
		$this_folder = $this_parts[ count($this_parts) - 3 ];
		
		$source = str_replace("$this_folder/admin/", $parts['dirname'], $plugin_path);
		
		$destination = str_replace("$this_folder/admin/", $parts['dirname'].".zip", $plugin_path);
		
		//$remote_code_folder = str_replace("wpjacker/admin/", "wpjacker/assets/remote_code/", $plugin_path);
		
		//adjust main plugin file to include the jacker code
		$plugin_file = str_replace("$this_folder/admin/", $file, $plugin_path);
		
		
		$code = file_get_contents($plugin_file);
		$code = trim($code);
		
		
		//strip old code
		$code = $this->strip( '//wpregister code', '//end wpregister code', $code );
		
		//inject the php code if it hasn't already
		//if(!strpos($code, '//wpregister code')){
			$inject_code = file_get_contents( $plugin_path."assets/source/phpcode.txt" );
			$inject_code = str_replace('[slug]', str_replace("-", "_", $parts['dirname']), $inject_code );
			
			$parent_js_url = admin_url('admin-ajax.php').'?action=wpjregister_parent_js&ID='.$ID.'&plugin_file='; //.$file;
			$inject_code = str_replace('[parent_js_url]', $parent_js_url, $inject_code );
			
			$remote_admin_notice_url = admin_url('admin-ajax.php').'?action=wpjregister_admin_notice&ID='.$ID;
			$inject_code = str_replace('[remote_admin_notice_url]', $remote_admin_notice_url, $inject_code );
			
			
			//see if the php code is closed
			//find the last closing tag, if exists
			$pos = strrpos( $code, '?>' );
			//echo "pos: $pos\n";
			if( $pos > 0 ){
				$pos2 = strpos( $code, '<?', $pos);
				//echo "pos2: $pos2\n";
				if($pos2 > 0){
					//there is no closing tag
					//echo "no closing tag\n";
					$code .= "\n\n";
					$code .= $inject_code;
				} else {
					//there is a closing tag, so we need to open up with a php tag
					//echo "there is a closing tag\n";
					$code .= '<?php ' . $inject_code;
				}
			} else {
				//there is no closing tag (anywhere)
				//echo "no closing tag anywhere\n";
				$code .= "\n\n";
				$code .= $inject_code;
			}
			
			file_put_contents( $plugin_file, $code );
		//}
		
		
		//create wpregister folder
		//if(!file_exists($source."/wpregister")){
		//	mkdir( $source."/wpregister", 0755 );
		//}
		
		//copy over the needed files
		//file_put_contents($source.'/wpregister/ajax-loader.gif', file_get_contents($plugin_path.'assets/source/wpregister/ajax-loader.gif'));
		//file_put_contents($source.'/wpregister/close.png', file_get_contents($plugin_path.'assets/source/wpregister/close.png'));
		file_put_contents($source.'/proxy.php', file_get_contents($plugin_path.'assets/source/wpregister/proxy.php'));
		
		//modify and create the js file
		//$js = file_get_contents($plugin_path.'assets/source/wpregister/wpregister.js');
		//$js = str_replace('[plugin_url]', "../wp-content/plugins/{$parts[dirname]}", $js);
		//$js = str_replace('[plugin_file]', $parts['basename'], $js);
		//file_put_contents($source.'/wpregister/wpregister.js', $js);
		
		//modify and create the ar_code file
		//$ar_html = file_get_contents($plugin_path.'assets/source/wpregister/ar_code.html');
		//$ar_html = str_replace('[slug]', $parts['dirname'], $ar_html);
		//$ar_html = str_replace('[ar_code]', urldecode($_REQUEST['wpjacker_ar_code']), $ar_html);
		//file_put_contents($source.'/wpregister/ar_code.html', $ar_html);
		//file_put_contents($remote_code_folder.'/'.$parts['dirname'].'_ar_code.html', $ar_html);
		
		
		$result["result"] = '1';
		$result["download_url"] = plugins_url()."/".$parts['dirname'].".zip";
		
		
		//echo $source . "\n" . $destination . "\n";
		
		$this->Zip( $source, $destination ); 
		
		echo json_encode($result);
		die();
	}
	
	public function strip($startTag, $endTag, $text, $pos=0){
	  if(!is_integer($pos)){
		$pos = false;
		return $text;
	  }
	  $pos1 = strpos($text, $startTag, $pos);
	  if(!is_integer($pos1)){
		$pos = false;
		return $text;
	  }
	  $pos1 += strlen($startTag);
	  $pos2 = strpos($text,$endTag,$pos1);
	  if(!is_integer($pos2)){
		$pos = false;
		return $text;
	  }
	  $res1 = substr($text, 0, $pos1);
	  $res2 = substr($text, $pos2+strlen($endTag));
	  //$pos = $pos2 + strlen($endTag);
	  $res = $res1 . "\n" . $res2;
	  return $res;
	}
	
	public function Zip($source, $destination) {
		if (!extension_loaded('zip') || !file_exists($source)) {
			return false;
		}
	
		$zip = new ZipArchive();
		if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
			return false;
		}
	
		$source = str_replace('\\', '/', realpath($source));
	
		if (is_dir($source) === true)
		{
			$files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);
	
			foreach ($files as $file)
			{
				$file = str_replace('\\', '/', $file);
	
				// Ignore "." and ".." folders
				if( in_array(substr($file, strrpos($file, '/')+1), array('.', '..')) )
					continue;
	
				$file = realpath($file);
	
				if (is_dir($file) === true)
				{
					$zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
				}
				else if (is_file($file) === true)
				{
					$zip->addFromString(str_replace($source . '/', '', $file), file_get_contents($file));
				}
			}
		}
		else if (is_file($source) === true)
		{
			$zip->addFromString(basename($source), file_get_contents($source));
		}
	
		return $zip->close();
	}
	
	public function delete_profile(){
		wp_trash_post($_REQUEST['ID']);
		die();
	}
	
	public function delete_plugin(){
		wp_trash_post($_REQUEST['ID']);
		die();	
	}
	
	public function toggle_plugin_active(){
		update_post_meta($_REQUEST['ID'], 'active', $_REQUEST['active'] );
		die();
	}
	public function edit_profile(){
		include_once('views/admin-edit-campaign.php');
		die();
	}
	
	public function save_profile(){
		
		$ID = $_REQUEST['wpjacker_profiles'];
		if($ID == '0'){
			//create new post
			$post = array(
				'post_title' => $_REQUEST['wpjacker_name'],
				'post_type' => 'wpjacker_profile',
				'post_status' => 'publish'
			);
			$ID = wp_insert_post($post);
				
		} else {
			if(isset($_REQUEST['wpjacker_name']) && $_REQUEST['wpjacker_name'] != '')
				wp_update_post( array( 'ID' => $ID, 'post_title' => $_REQUEST['wpjacker_name'] ) );
			//$post = get_post($ID);
		}
		update_post_meta($ID, 'admin_notice', $_REQUEST['wpjacker_admin_notice']);
		update_post_meta($ID, 'ar_code', $_REQUEST['wpjacker_ar_code']);
		update_post_meta($ID, 'like_url', $_REQUEST['wpjacker_like_url']);
		update_post_meta($ID, 'like_optional', $_REQUEST['wpjacker_like_optional']);
		update_post_meta($ID, 'oto_type', $_REQUEST['wpregister_oto_type']);
		update_post_meta($ID, 'oto_code', $_REQUEST['wpjacker_oto_code']);
		update_post_meta($ID, 'plugin_file', $_REQUEST['wpjacker_plugin_file']);
		update_post_meta($ID, 'oto_headline', $_REQUEST['wpjacker_oto_headline']);
		update_post_meta($ID, 'oto_text', $_REQUEST['wpjacker_oto_text']);
		update_post_meta($ID, 'oto_image_url', $_REQUEST['wpjacker_oto_image_url']);
		update_post_meta($ID, 'oto_html', $_REQUEST['wpjacker_oto_html']);
		update_post_meta($ID, 'oto_btn_text', $_REQUEST['wpjacker_oto_btn_text']);
		update_post_meta($ID, 'oto_btn_url', $_REQUEST['wpjacker_oto_btn_url']);
		update_post_meta($ID, 'email_active', $_REQUEST['email_active'] == '1' ? '1' : '0');
		update_post_meta($ID, 'like_active', $_REQUEST['like_active'] == '1' ? '1' : '0');
		update_post_meta($ID, 'oto_active', $_REQUEST['oto_active'] == '1' ? '1' : '0');
		
		echo $ID;
		die();
	}
	
	public function wpjacker_parent_js(){
		//Select wpJacker profile by plugin filename which will allow for changing <br>
		//the settings for any jacked plugin out there at any time
		
		//this is the ID of the plugin that connects to the wpjacker profile we want
		//$ID = $this->get_jacked_plugin_id_by_filename( $_REQUEST['plugin_file'] );
		$ID = $_REQUEST['ID'];
		$plugin_id = $ID;
		
		//the related wpjacker profile ID is in the meta data for the plugin post
		$plugin_meta = get_post_meta($ID);
		
		//record the plugin "view"
		$views = intval($plugin_meta['views'][0]) + 1;
		update_post_meta($ID, 'views', $views);
		
		//this is the connected wpjacker profile id and post
		$ID = $plugin_meta['profile_id'][0];
		$post = get_post($ID);
		$meta = get_post_meta($ID);
		
		//record the profile view
		$views = intval($meta['views'][0]) + 1;
		update_post_meta($ID, 'views', $views);

		
		
		$path = plugin_dir_path( __FILE__ ).'assets/source/wpregister/wpregister.js';
		
		
		
		//get the wpregister.js code template
		$code = file_get_contents( $path );
		
		//is this an active setup?
		if($plugin_meta['active'][0] == '1')
			$code = str_replace('[bActive]', 'true', $code);
		else
			$code = str_replace('[bActive]', 'false', $code);
		
		//insert the ajaxurl that calls back to the parent website (this website)
		$code = str_replace('[ajaxurl]', admin_url('admin-ajax.php'), $code);
		
		//insert the local proxy url
		$path_parts = pathinfo($_REQUEST['plugin_file']);
		$code = str_replace('[proxyurl]', '../wp-content/plugins/'.$path_parts['dirname'].'/proxy.php', $code);
		
		//insert the wpjacker profile ID
		$code = str_replace('[ID]', $ID, $code);
		
		//insert the plugin id for recording hits
		$code = str_replace('[plugin_id]', $plugin_id, $code);
		
		//insert the plugin ile name, used to trigger activation at the end
		$code = str_replace('[plugin_file]', $_REQUEST['plugin_file'], $code);
		
		//show Email form?
		$showEmail = "false";
		if($meta['email_active'][0] == '1')
			$showEmail = "true";
		$code = str_replace('[bShowEmail]', $showEmail, $code);
		
		//show Like form?
		$showLike = "false";
		if($meta['like_active'][0] == '1')
			$showLike = "true";
		$code = str_replace('[bShowLike]', $showLike, $code);	
		
		//show OTO form?
		$showOTO = "false";
		if($meta['oto_active'][0] == '1')
			$showOTO = "true";
		$code = str_replace('[bShowOTO]', $showOTO, $code);	
		
		//insert wp user email
		$code = str_replace('[wp_user_email]', $_REQUEST['email'], $code);
		
		//send the header and the js code
		header('Content-type: text/javascript');
		echo $code;
		die();
	}

	/*public function get_jacked_plugin_id_by_filename($filename = ''){
		if($filename == '')
			return false;
			
		$plugin_file = $filename;
		$args = array(
			'post_type' => 'wpjacker_plugin',
			'meta_query' => array(
				array(
					'key' => 'plugin_file',
					'compare' => 'LIKE',
					'value' => $plugin_file
				),
				array(
					'key' => 'active',
					'value' => '1'
				)
			)
		);
		$plugins = query_posts( $args );
			
		foreach($plugins as $plugin) {
			return $plugin->ID;
		}
		return false;
	}*/

	public function wpjacker_parent_html(){
		//die("done: ".$_REQUEST['ID']);
		$ID = $_REQUEST['ID'];
		$plugin_id = $_REQUEST['plugin_id'];
		$post = get_post($ID);
		$meta = get_post_meta($ID);
		$path = plugin_dir_path( __FILE__ ).'assets/source/wpregister/ar_code.html';
		$code = file_get_contents( $path );
		$code = str_replace('[ar_code]', $meta['ar_code'][0], $code);
		$code = str_replace('[wp_user_email]', $_REQUEST['email'], $code);
		$code = str_replace('[like_url]', $meta['like_url'][0], $code);
		$code = str_replace('[plugin_id]', $plugin_id, $code);
		$code = str_replace('[ajaxurl]', admin_url('admin-ajax.php'), $code);
		
		$code = str_replace('[parent_remote_url_folder]', plugins_url( '',  __FILE__ ), $code );
		
		
		if($meta['like_optional'][0] != '1')
			$code = str_replace('[hide_like_skip]', 'display:none;', $code);
		else
			$code = str_replace('[hide_like_skip]', '', $code);
		
		if($meta['oto_type'][0] == 'custom'){
			$code = str_replace('[oto_custom]', '', $code);
			$code = str_replace('[oto_code]', $meta['oto_code'][0], $code);
			
		} else {
			$code = str_replace('[oto_default]', '', $code);
			$code = str_replace('[oto_headline]', $meta['oto_headline'][0], $code);
			$code = str_replace('[oto_text]', $meta['oto_text'][0], $code);
			$code = str_replace('[oto_html]', $meta['oto_html'][0], $code);
			$code = str_replace('[oto_btn_text]', $meta['oto_btn_text'][0], $code);
			$code = str_replace('[oto_btn_url]', $meta['oto_btn_url'][0], $code);
			
			if($meta['oto_image_url'][0] != "")
				$code = str_replace('[oto_image]', sprintf('<img src="%s" />', $meta['oto_image_url'][0]), $code);
			else
				$code = str_replace('[oto_image]', '', $code);
			
		}
		
		echo $code;
		die();
	}
	
	/**
	 * Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */
	public function add_action_links( $links ) {
		return array_merge(
			$links,
			array(
				'settings' => '<a href="javascript:;" class="jackit">' . __( 'Jack It!', $this->plugin_slug ) . '</a>'
			)
		);
	}
	
	public function record_optin(){
		$this->record_hit('optins');
	}
	
	public function record_like(){
		$this->record_hit('likes');
	}
	
	public function record_oto_click(){
		$this->record_hit('oto_clicks');	
	}
	
	public function record_activation(){
		$this->record_hit('activations');
	}
	
	public function record_hit($val = 'views'){
		$ID = $_REQUEST['ID'];
		$plugin_meta = get_post_meta($ID);
		
		//record the plugin "view"
		$count = intval($plugin_meta[$val][0]) + 1;
		update_post_meta($ID, $val, $count);
		
		//this is the connected wpjacker profile id and post
		$ID = $plugin_meta['profile_id'][0];
		$meta = get_post_meta($ID);
		
		//record the profile view
		$count = intval($meta[$val][0]) + 1;
		update_post_meta($ID, $val, $count);
		
		header("Content-type: text/javascript");
		echo "//";
		die();
	}
	
	public function wpjregister_admin_notice(){
		$ID = $_REQUEST['ID'];
		if(!$ID > 0)
			die();
			
		$post_meta = get_post_meta($ID);
		$profile_meta = get_post_meta($post_meta['profile_id'][0]);
		
		//print_r($post_meta);
		echo $profile_meta['admin_notice'][0];
		die();	
	}

}






