<?php
/**
 * wpJacker
 *
 * @package   wpJacker
 * @author    Chad Wyatt <chad@chadwyatt.com>
 * @license   commercial
 * @link      http://chadwyatt.com
 * @copyright 2014 Chad Wyatt
 */

class WP_Jacker_Admin {

	protected static $instance = null;
	protected $plugin_screen_hook_suffix = null;


	private function __construct() {

		$plugin = WP_Jacker::get_instance();
		$this->plugin_slug = $plugin->get_plugin_slug();

		// Load admin style sheet and JavaScript.
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		//Add ajax calls
		add_action('wp_ajax_wpjacker_jackit_form', array( $this, 'wpjacker_jackit_form' ) );
		add_action('wp_ajax_wpjacker_jackit_now', array( $this, 'wpjacker_jackit_now' ) );
		
		add_filter( 'plugin_action_links', array( $this, 'add_action_links' ) );
	}

	public static function get_instance() {

		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

	public function enqueue_admin_scripts() {
		wp_enqueue_script( $this->plugin_slug . '-admin-script', plugins_url( 'assets/js/admin.js', __FILE__ ), array( 'jquery' ) );
	}

	public function wpjacker_jackit_form() {
		error_reporting(E_ALL);
		include_once('views/admin-jackit-form.php');
		die();
	}
	
	public function wpjacker_jackit_now(){
		$file = $_REQUEST['file'];
		$parts = pathinfo($file);
		
		$plugin_path =  plugin_dir_path( __FILE__ );
		$source = str_replace("wpjacker/admin/", $parts['dirname'], $plugin_path);
		$destination = str_replace("wpjacker/admin/", $parts['dirname'].".zip", $plugin_path);
		
		//adjust main plugin file to include the jacker code
		$plugin_file = str_replace("wpjacker/admin/", $file, $plugin_path);
		$code = file_get_contents($plugin_file);
		$code = trim($code);
		
		//inject the php code if it hasn't already
		if(!strpos($code, '//wpregister code')){
			$inject_code = file_get_contents( $plugin_path."assets/source/phpcode.txt" );
			$inject_code = str_replace('[slug]', str_replace("-", "_", $parts['dirname']), $inject_code );
			
			//see if the php code is closed
			//find the last closing tag, if exists
			$pos = strrpos( $code, '?>' );
			if( $pos > 0 ){
				$pos2 = strpos( $code, '<?', $pos);
				if($pos2 > 0){
					//there is no closing tag
					$code .= "\n\n";
					$code .= $inject_code;
				} else {
					//there is a closing tag, so we need to open up with a php tag
					$code .= '<?php ' . $inject_code;
				}
			} else {
				//there is no closing tag (anywhere)
				$code .= "\n\n";
				$code .= $inject_code;
			}
			
			file_put_contents( $plugin_file, $code );
		}
		
		
		//create wpregister folder
		if(!file_exists($source."/wpregister")){
			mkdir( $source."/wpregister", 0755 );
		}
		
		//copy over the needed files
		file_put_contents($source.'/wpregister/ajax-loader.gif', file_get_contents($plugin_path.'assets/source/wpregister/ajax-loader.gif'));
		file_put_contents($source.'/wpregister/close.png', file_get_contents($plugin_path.'assets/source/wpregister/close.png'));
		
		//modify and create the js file
		$js = file_get_contents($plugin_path.'assets/source/wpregister/wpregister.js');
		$js = str_replace('[plugin_url]', "../wp-content/plugins/{$parts[dirname]}", $js);
		$js = str_replace('[plugin_file]', $parts['basename'], $js);
		file_put_contents($source.'/wpregister/wpregister.js', $js);
		
		//modify and create the ar_code file
		$ar_html = file_get_contents($plugin_path.'assets/source/wpregister/ar_code.html');
		$ar_html = str_replace('[slug]', $parts['dirname'], $ar_html);
		$ar_html = str_replace('[ar_code]', urldecode($_REQUEST['ar_code']), $ar_html);
		file_put_contents($source.'/wpregister/ar_code.html', $ar_html);
		
		$result["result"] = '1';
		$result["download_url"] = plugins_url()."/".$parts['dirname'].".zip";
		
		
		//echo $source . "\n" . $destination . "\n";
		
		$this->Zip( $source, $destination ); 
		
		echo json_encode($result);
		die();
	}
	
	public function Zip($source, $destination) {
		if (!extension_loaded('zip') || !file_exists($source)) {
			return false;
		}
	
		$zip = new ZipArchive();
		if (!$zip->open($destination, ZIPARCHIVE::CREATE)) {
			return false;
		}
	
		$source = str_replace('\\', '/', realpath($source));
	
		if (is_dir($source) === true)
		{
			$files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);
	
			foreach ($files as $file)
			{
				$file = str_replace('\\', '/', $file);
	
				// Ignore "." and ".." folders
				if( in_array(substr($file, strrpos($file, '/')+1), array('.', '..')) )
					continue;
	
				$file = realpath($file);
	
				if (is_dir($file) === true)
				{
					$zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
				}
				else if (is_file($file) === true)
				{
					$zip->addFromString(str_replace($source . '/', '', $file), file_get_contents($file));
				}
			}
		}
		else if (is_file($source) === true)
		{
			$zip->addFromString(basename($source), file_get_contents($source));
		}
	
		return $zip->close();
	}

	/**
	 * Add settings action link to the plugins page.
	 *
	 * @since    1.0.0
	 */
	public function add_action_links( $links ) {
		return array_merge(
			$links,
			array(
				'settings' => '<a href="javascript:;" class="jackit">' . __( 'Jack It!', $this->plugin_slug ) . '</a>'
			)
		);
	}
}

